package factory;

/*
 * functional module Factory is
 *   local import from Controller using all
 *   local import from Producer
 *   local import from Consumer
 */
public class Factory {

    private static Producer p = new Producer();
    private static Consumer c = new Consumer();

    /*
     * functional module Controller is 
     *   local import from Producer using start, stop 
     *   local import from Consumer using start, stop
     */
    static class Controller {
        public static void startupFactory() {
            p.start();
            c.start();
        }

        public static void shutdownFactory() {
            p.stop();
            c.stop();
        }
    }

    /*
     * functional module Producer is Task
     *   general import from Item using all 
     *   general import from Buffer using write
     */
    static class Producer extends Thread
    {
        public void run() {
            while (true) {
                produce();
            }
        }

        private void produce() {
            try {
                Thread.sleep((long) (Math.random() * 500));
                Item item = new Item();
                System.out.println("P: Produced item with timestamp " 
                        + item.getTimestamp());
                Buffer.write(item);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    /*
     * functional module Consumer is Task
     *   general import from Item using getTimestamp
     *   general import from Buffer using read
     */
    static class Consumer extends Thread
    {
        public void run() {
            while (true) {
                consume();
            }
        }

        private void consume() {
            try {
                Thread.sleep((long) (Math.random() * 1000));
                Item item = Buffer.read();
                System.out.println("C: Consumed item with timestamp " + item.getTimestamp());
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

    }

    public static void main(String[] args) {
        Controller.startupFactory();
        try {
            Thread.sleep(15000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        Controller.shutdownFactory();
    }
}
